------------------------------------------------------------------------------
--                                                                          --
--                           POLYORB COMPONENTS                             --
--                                                                          --
--      D Y N A M I C A N Y . D Y N A N Y F A C T O R Y . H E L P E R       --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--         Copyright (C) 2005-2006, Free Software Foundation, Inc.          --
--                                                                          --
-- PolyORB is free software; you  can  redistribute  it and/or modify it    --
-- under terms of the  GNU General Public License as published by the  Free --
-- Software Foundation;  either version 2,  or (at your option)  any  later --
-- version. PolyORB is distributed  in the hope that it will be  useful,    --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License  for more details.  You should have received  a copy of the GNU  --
-- General Public License distributed with PolyORB; see file COPYING. If    --
-- not, write to the Free Software Foundation, 51 Franklin Street, Fifth    --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
--
--
--
--
--
--
--
--                  PolyORB is maintained by AdaCore                        --
--                     (email: sales@adacore.com)                           --
--                                                                          --
------------------------------------------------------------------------------

with PolyORB.Exceptions;
with PolyORB.Initialization;
with PolyORB.Utils.Strings;

package body DynamicAny.DynAnyFactory.Helper is

   procedure Deferred_Initialization;

   procedure Raise_InconsistentTypeCode_From_Any (Item : PolyORB.Any.Any);
   pragma No_Return (Raise_InconsistentTypeCode_From_Any);

   -----------------------------
   -- Deferred_Initialization --
   -----------------------------

   procedure Deferred_Initialization is
   begin
      declare
         Name : constant CORBA.String
           := CORBA.To_CORBA_String ("DynAnyFactory");
         Id   : constant CORBA.String
           := CORBA.To_CORBA_String
           ("IDL:omg.org/DynamicAny/DynAnyFactory:1.0");

      begin
         CORBA.TypeCode.Internals.Add_Parameter
           (TC_DynAnyFactory, CORBA.To_Any (Name));
         CORBA.TypeCode.Internals.Add_Parameter
           (TC_DynAnyFactory, CORBA.To_Any (Id));
      end;

      declare
         Name : CORBA.String := CORBA.To_CORBA_String ("InconsistentTypeCode");
         Id   : CORBA.String
           := CORBA.To_CORBA_String
           ("IDL:omg.org/DynamicAny/DynAnyFactory/InconsistentTypeCode:1.0");

      begin
         CORBA.TypeCode.Internals.Add_Parameter
           (TC_InconsistentTypeCode, CORBA.To_Any (Name));
         CORBA.TypeCode.Internals.Add_Parameter
           (TC_InconsistentTypeCode, CORBA.To_Any (Id));
      end;
      PolyORB.Exceptions.Register_Exception
        (CORBA.TypeCode.Internals.To_PolyORB_Object (TC_InconsistentTypeCode),
         Raise_InconsistentTypeCode_From_Any'Access);
   end Deferred_Initialization;

   --------------
   -- From_Any --
   --------------

   function From_Any
     (Item : CORBA.Any)
      return InconsistentTypeCode_Members
   is
      pragma Unreferenced (Item);

      Result : InconsistentTypeCode_Members;

   begin
      return Result;
   end From_Any;

   --------------------------------
   -- Raise_InconsistentTypeCode --
   --------------------------------

   procedure Raise_InconsistentTypeCode
     (Members : InconsistentTypeCode_Members)
   is
   begin
      PolyORB.Exceptions.User_Raise_Exception
        (InconsistentTypeCode'Identity,
         Members);
   end Raise_InconsistentTypeCode;

   -----------------------------------------
   -- Raise_InconsistentTypeCode_From_Any --
   -----------------------------------------

   procedure Raise_InconsistentTypeCode_From_Any (Item : PolyORB.Any.Any) is
      Members : constant InconsistentTypeCode_Members
        := From_Any (CORBA.Internals.To_CORBA_Any (Item));

   begin
      PolyORB.Exceptions.User_Raise_Exception
        (InconsistentTypeCode'Identity,
         Members);
   end Raise_InconsistentTypeCode_From_Any;

   ------------
   -- To_Any --
   ------------

   function To_Any (Item : InconsistentTypeCode_Members) return CORBA.Any is
      pragma Unreferenced (Item);

      Result : CORBA.Any
        := CORBA.Internals.Get_Empty_Any_Aggregate (TC_InconsistentTypeCode);

   begin
      return Result;
   end To_Any;

   ------------------
   -- To_Local_Ref --
   ------------------

   function To_Local_Ref
     (The_Ref : CORBA.Object.Ref'Class)
      return Local_Ref
   is
   begin
      if CORBA.Object.Is_Nil (The_Ref)
        or else CORBA.Object.Is_A (The_Ref, Repository_Id)
      then
         return Unchecked_To_Local_Ref (The_Ref);
      end if;

      CORBA.Raise_Bad_Param (CORBA.Default_Sys_Member);
   end To_Local_Ref;

   ----------------------------
   -- Unchecked_To_Local_Ref --
   ----------------------------

   function Unchecked_To_Local_Ref
     (The_Ref : CORBA.Object.Ref'Class)
      return Local_Ref
   is
      Result : DynamicAny.DynAnyFactory.Local_Ref;

   begin
      Set (Result, CORBA.Object.Object_Of (The_Ref));
      return Result;
   end Unchecked_To_Local_Ref;

begin
   declare
      use PolyORB.Initialization;
      use PolyORB.Initialization.String_Lists;
      use PolyORB.Utils.Strings;

   begin
      Register_Module
        (Module_Info'
         (Name      => +"DynamicAny.DynAnyFactory.Helper",
          Conflicts => Empty,
          Depends   => +"any"
          & "exceptions",
          Provides  => Empty,
          Implicit  => False,
          Init      => Deferred_Initialization'Access));
   end;
end DynamicAny.DynAnyFactory.Helper;
