------------------------------------------------------------------------------
--                                                                          --
--                           POLYORB COMPONENTS                             --
--                                                                          --
--               D Y N A M I C A N Y . D Y N S E Q U E N C E                --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--         Copyright (C) 2005-2006, Free Software Foundation, Inc.          --
--                                                                          --
-- PolyORB is free software; you  can  redistribute  it and/or modify it    --
-- under terms of the  GNU General Public License as published by the  Free --
-- Software Foundation;  either version 2,  or (at your option)  any  later --
-- version. PolyORB is distributed  in the hope that it will be  useful,    --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License  for more details.  You should have received  a copy of the GNU  --
-- General Public License distributed with PolyORB; see file COPYING. If    --
-- not, write to the Free Software Foundation, 51 Franklin Street, Fifth    --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
--
--
--
--
--
--
--
--                  PolyORB is maintained by AdaCore                        --
--                     (email: sales@adacore.com)                           --
--                                                                          --
------------------------------------------------------------------------------

with CORBA.Object;

with DynamicAny.DynSequence.Impl;

package body DynamicAny.DynSequence is

   ------------------
   -- Get_Elements --
   ------------------

   function Get_Elements (Self : Local_Ref) return AnySeq is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      return Impl.Get_Elements (Impl.Object_Ptr (Entity_Of (Self)));
   end Get_Elements;

   -----------------------------
   -- Get_Elements_As_Dyn_Any --
   -----------------------------

   function Get_Elements_As_Dyn_Any (Self : Local_Ref) return DynAnySeq is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      return Impl.Get_Elements_As_Dyn_Any (Impl.Object_Ptr (Entity_Of (Self)));
   end Get_Elements_As_Dyn_Any;

   ----------------
   -- Get_Length --
   ----------------

   function Get_Length (Self : Local_Ref) return CORBA.Unsigned_Long is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      return Impl.Get_Length (Impl.Object_Ptr (Entity_Of (Self)));
   end Get_Length;

   ------------------
   -- Set_Elements --
   ------------------

   procedure Set_Elements
     (Self  : Local_Ref;
      Value : AnySeq)
   is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      Impl.Set_Elements (Impl.Object_Ptr (Entity_Of (Self)), Value);
   end Set_Elements;

   -----------------------------
   -- Set_Elements_As_Dyn_Any --
   -----------------------------

   procedure Set_Elements_As_Dyn_Any
     (Self  : Local_Ref;
      Value : DynAnySeq)
   is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      Impl.Set_Elements_As_Dyn_Any (Impl.Object_Ptr (Entity_Of (Self)), Value);
   end Set_Elements_As_Dyn_Any;

   ----------------
   -- Set_Length --
   ----------------

   procedure Set_Length
     (Self : Local_Ref;
      Len  : CORBA.Unsigned_Long)
   is
      Self_Ref : constant CORBA.Object.Ref := CORBA.Object.Ref (Self);

   begin
      if CORBA.Object.Is_Nil (Self_Ref) then
         CORBA.Raise_Inv_Objref (CORBA.Default_Sys_Member);
      end if;

      Impl.Set_Length (Impl.Object_Ptr (Entity_Of (Self)), Len);
   end Set_Length;

end DynamicAny.DynSequence;
