------------------------------------------------------------------------------
--                                                                          --
--                           POLYORB COMPONENTS                             --
--                                                                          --
--           P O L Y O R B . C O R B A _ P . E X C E P T I O N S            --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--         Copyright (C) 2001-2006, Free Software Foundation, Inc.          --
--                                                                          --
-- PolyORB is free software; you  can  redistribute  it and/or modify it    --
-- under terms of the  GNU General Public License as published by the  Free --
-- Software Foundation;  either version 2,  or (at your option)  any  later --
-- version. PolyORB is distributed  in the hope that it will be  useful,    --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License  for more details.  You should have received  a copy of the GNU  --
-- General Public License distributed with PolyORB; see file COPYING. If    --
-- not, write to the Free Software Foundation, 51 Franklin Street, Fifth    --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
--
--
--
--
--
--
--
--                  PolyORB is maintained by AdaCore                        --
--                     (email: sales@adacore.com)                           --
--                                                                          --
------------------------------------------------------------------------------

--  Exceptions management for the CORBA application personality of PolyORB.

with Ada.Exceptions;

with PolyORB.Any;
with PolyORB.Errors;

package PolyORB.CORBA_P.Exceptions is

   procedure Raise_From_Any
     (Occurrence : PolyORB.Any.Any);
   pragma No_Return (Raise_From_Any);
   --  Raise CORBA exception from data in 'Occurrence'.

   function System_Exception_To_Any
     (E : Ada.Exceptions.Exception_Occurrence)
     return PolyORB.Any.Any;
   --  Convert a CORBA System Exception into a Any.

   procedure Raise_From_Error
     (Error : in out PolyORB.Errors.Error_Container);
   pragma No_Return (Raise_From_Error);
   --  Raise a CORBA specific exception from the data in 'Error'

   --  Exceptions classification

   function Is_Forward_Request
     (Occurrence : PolyORB.Any.Any)
     return Boolean;
   --  Return True iff Occurrence is a PolyORB forward request exception

   function Is_System_Exception
     (Occurrence : PolyORB.Any.Any)
      return Boolean;
   --  Return True iff Occurrence is an ORB system exception

   ----------------------------
   -- Raise_From_Error Hooks --
   ----------------------------

   --  The CORBA aplication personality may raise exceptions found in
   --  different packages. Hooks are set up to work around circular
   --  references problems.

   type Raise_From_Error_Hook is access
     procedure (Error : in out PolyORB.Errors.Error_Container);

   CORBA_Raise_From_Error      : Raise_From_Error_Hook := null;
   --  Raise CORBA.* exceptions

   POA_Raise_From_Error        : Raise_From_Error_Hook := null;
   --  Raise PortableServer.POA.* exceptions

   POAManager_Raise_From_Error : Raise_From_Error_Hook := null;
   --  Raise PortableServer.POAManager.* exceptions

end PolyORB.CORBA_P.Exceptions;
