------------------------------------------------------------------------------
--                                                                          --
--                           POLYORB COMPONENTS                             --
--                                                                          --
--                  PORTABLEINTERCEPTOR.CLIENTREQUESTINFO                   --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--         Copyright (C) 2004-2006, Free Software Foundation, Inc.          --
--                                                                          --
-- This specification is derived from the CORBA Specification, and adapted  --
-- for use with PolyORB. The copyright notice above, and the license        --
-- provisions that follow apply solely to the contents neither explicitely  --
-- nor implicitely specified by the CORBA Specification defined by the OMG. --
--                                                                          --
-- PolyORB is free software; you  can  redistribute  it and/or modify it    --
-- under terms of the  GNU General Public License as published by the  Free --
-- Software Foundation;  either version 2,  or (at your option)  any  later --
-- version. PolyORB is distributed  in the hope that it will be  useful,    --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License  for more details.  You should have received  a copy of the GNU  --
-- General Public License distributed with PolyORB; see file COPYING. If    --
-- not, write to the Free Software Foundation, 51 Franklin Street, Fifth    --
-- Floor, Boston, MA 02111-1301, USA.                                       --
--                                                                          --
--
--
--
--
--
--
--
--                  PolyORB is maintained by AdaCore                        --
--                     (email: sales@adacore.com)                           --
--                                                                          --
------------------------------------------------------------------------------

with CORBA.Object;
with CORBA.Policy;

with IOP;
with PortableInterceptor.RequestInfo;

package PortableInterceptor.ClientRequestInfo is

   type Local_Ref is
      new PortableInterceptor.RequestInfo.Local_Ref with null record;

   function Get_Target
     (Self : Local_Ref)
      return CORBA.Object.Ref;

   function Get_Effective_Target
     (Self : Local_Ref)
      return CORBA.Object.Ref;

   function Get_Effective_Profile
     (Self : Local_Ref)
      return IOP.TaggedProfile;

   function Get_Received_Exception
     (Self : Local_Ref)
      return CORBA.Any;

   function Get_Received_Exception_Id
     (Self : Local_Ref)
      return CORBA.RepositoryId;

   function Get_Effective_Component
     (Self : Local_Ref;
      Id   : IOP.ComponentId)
      return IOP.TaggedComponent;

   function Get_Effective_Components
     (Self : Local_Ref;
      Id   : IOP.ComponentId)
      return IOP.TaggedComponentSeq;

   function Get_Request_Policy
     (Self     : Local_Ref;
      IDL_Type : CORBA.PolicyType)
      return CORBA.Policy.Ref;

   procedure Add_Request_Service_Context
     (Self            : Local_Ref;
      Service_Context : IOP.ServiceContext;
      Replace         : CORBA.Boolean);

   --  Repository_Ids

   ClientRequestInfo_Root_Repository_Id : constant Standard.String
     := "IDL:omg.org/PortableInterceptor/ClientRequestInfo";

   Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & ":1.0";

   Target_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/target:1.0";

   Effective_Target_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/effective_target:1.0";

   Effective_Profile_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/effective_profile:1.0";

   Received_Exception_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/received_exception:1.0";

   Received_Exception_Id_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/received_exception_id:1.0";

   Get_Effective_Component_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/get_effective_component:1.0";

   Get_Effective_Components_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/get_effective_components:1.0";

   Get_Request_Policy_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id & "/get_request_policy:1.0";

   Add_Request_Service_Context_Repository_Id : constant Standard.String
     := ClientRequestInfo_Root_Repository_Id
     & "/add_request_service_context:1.0";

end PortableInterceptor.ClientRequestInfo;
