------------------------------------------------------------------------------
--                                                                          --
--                           POLYORB COMPONENTS                             --
--                                                                          --
--                  POLYORB.SMART_POINTERS.INITIALIZATION                   --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--            Copyright (C) 2004 Free Software Foundation, Inc.             --
--                                                                          --
-- PolyORB is free software; you  can  redistribute  it and/or modify it    --
-- under terms of the  GNU General Public License as published by the  Free --
-- Software Foundation;  either version 2,  or (at your option)  any  later --
-- version. PolyORB is distributed  in the hope that it will be  useful,    --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License  for more details.  You should have received  a copy of the GNU  --
-- General Public License distributed with PolyORB; see file COPYING. If    --
-- not, write to the Free Software Foundation, 59 Temple Place - Suite 330, --
-- Boston, MA 02111-1307, USA.                                              --
--                                                                          --
--
--
--
--
--
--
--
--                PolyORB is maintained by ACT Europe.                      --
--                    (email: sales@act-europe.fr)                          --
--                                                                          --
------------------------------------------------------------------------------

--  Initialization code for PolyORB.Smart_Pointers

with Ada.Tags;

with PolyORB.Initialization;
pragma Elaborate_All (PolyORB.Initialization); --  WAG:3.15

with PolyORB.Tasking.Mutexes;
with PolyORB.Utils.Strings;

package body PolyORB.Smart_Pointers.Initialization is

   ------------------------------------
   -- Debugging hooks implementation --
   ------------------------------------

   function Entity_External_Tag
     (X : Unsafe_Entity'Class)
      return String;
   function Ref_External_Tag
     (X : Ref'Class)
      return String;
   --  Return the external representation of X'Tag.

   function Entity_External_Tag
     (X : Unsafe_Entity'Class)
      return String is
   begin
      return Ada.Tags.External_Tag (X'Tag);
   end Entity_External_Tag;

   function Ref_External_Tag
     (X : Ref'Class)
      return String is
   begin
      return Ada.Tags.External_Tag (X'Tag);
   end Ref_External_Tag;

   ----------------
   -- Initialize --
   ----------------

   procedure Initialize;

   procedure Initialize is
   begin
      Tasking.Mutexes.Create (Counter_Lock);
      Smart_Pointers.Entity_External_Tag := Entity_External_Tag'Access;
      Smart_Pointers.Ref_External_Tag    := Ref_External_Tag'Access;
   end Initialize;

   use PolyORB.Initialization;
   use PolyORB.Initialization.String_Lists;
   use PolyORB.Utils.Strings;

begin
   Register_Module
     (Module_Info'
      (Name      => +"smart_pointers",
       Conflicts => Empty,
       Depends   => +"tasking.mutexes",
       Provides  => Empty,
       Implicit  => False,
       Init      => Initialize'Access));
end PolyORB.Smart_Pointers.Initialization;
